#version 130

//Shader by LoLip_p

#include "/settings.glsl"

varying vec2 TexCoords;

uniform float viewWidth, viewHeight;
uniform sampler2D colortex4;

ivec2 texelCoord = ivec2(gl_FragCoord.xy);

float quality[12] = float[12] (1.0, 1.0, 1.0, 1.0, 1.0, 1.5, 2.0, 2.0, 2.0, 2.0, 4.0, 8.0);

float GetLuminance(vec3 color)
{
    return dot(color, vec3(0.299, 0.587, 0.114));
}

void FXAA311(inout vec3 color)
{
    float edgeThresholdMin = 0.03125;
    float edgeThresholdMax = 0.125;
    float subpixelQuality = 0.75;
    int iterations = 12;

    vec2 view = 1.0 / vec2(viewWidth, viewHeight);

    float lumaCenter = GetLuminance(color);

    float lumaDown  = GetLuminance(pow(texelFetch(colortex4, texelCoord + ivec2( 0, -1), 0).rgb, vec3(1.0f / 2.2f)));
    float lumaUp    = GetLuminance(pow(texelFetch(colortex4, texelCoord + ivec2( 0,  1), 0).rgb, vec3(1.0f / 2.2f)));
    float lumaLeft  = GetLuminance(pow(texelFetch(colortex4, texelCoord + ivec2(-1,  0), 0).rgb, vec3(1.0f / 2.2f)));
    float lumaRight = GetLuminance(pow(texelFetch(colortex4, texelCoord + ivec2( 1,  0), 0).rgb, vec3(1.0f / 2.2f)));

    float lumaMin = min(lumaCenter, min(min(lumaDown, lumaUp), min(lumaLeft, lumaRight)));
    float lumaMax = max(lumaCenter, max(max(lumaDown, lumaUp), max(lumaLeft, lumaRight)));

    float lumaRange = lumaMax - lumaMin;

    if (lumaRange > max(edgeThresholdMin, lumaMax * edgeThresholdMax))
	{
        float lumaDownLeft  = GetLuminance(pow(texelFetch(colortex4, texelCoord + ivec2(-1, -1), 0).rgb, vec3(1.0f / 2.2f)));
        float lumaUpRight   = GetLuminance(pow(texelFetch(colortex4, texelCoord + ivec2( 1,  1), 0).rgb, vec3(1.0f / 2.2f)));
        float lumaUpLeft    = GetLuminance(pow(texelFetch(colortex4, texelCoord + ivec2(-1,  1), 0).rgb, vec3(1.0f / 2.2f)));
        float lumaDownRight = GetLuminance(pow(texelFetch(colortex4, texelCoord + ivec2( 1, -1), 0).rgb, vec3(1.0f / 2.2f)));

        float lumaDownUp    = lumaDown + lumaUp;
        float lumaLeftRight = lumaLeft + lumaRight;

        float lumaLeftCorners  = lumaDownLeft  + lumaUpLeft;
        float lumaDownCorners  = lumaDownLeft  + lumaDownRight;
        float lumaRightCorners = lumaDownRight + lumaUpRight;
        float lumaUpCorners    = lumaUpRight   + lumaUpLeft;

        float edgeHorizontal = abs(-2.0 * lumaLeft   + lumaLeftCorners ) +
                               abs(-2.0 * lumaCenter + lumaDownUp      ) * 2.0 +
                               abs(-2.0 * lumaRight  + lumaRightCorners);
        float edgeVertical   = abs(-2.0 * lumaUp     + lumaUpCorners   ) +
                               abs(-2.0 * lumaCenter + lumaLeftRight   ) * 2.0 +
                               abs(-2.0 * lumaDown   + lumaDownCorners );

        bool isHorizontal = (edgeHorizontal >= edgeVertical);

        float luma1 = isHorizontal ? lumaDown : lumaLeft;
        float luma2 = isHorizontal ? lumaUp : lumaRight;
        float gradient1 = luma1 - lumaCenter;
        float gradient2 = luma2 - lumaCenter;

        bool is1Steepest = abs(gradient1) >= abs(gradient2);
        float gradientScaled = 0.25 * max(abs(gradient1), abs(gradient2));

        float stepLength = isHorizontal ? view.y : view.x;

        float lumaLocalAverage = 0.0;

        if (is1Steepest)
		{
            stepLength = - stepLength;
            lumaLocalAverage = 0.5 * (luma1 + lumaCenter);
        }
		else
		{
            lumaLocalAverage = 0.5 * (luma2 + lumaCenter);
        }

        vec2 currentUv = TexCoords;
        if (isHorizontal)
		{
            currentUv.y += stepLength * 0.5;
        }
		else
		{
            currentUv.x += stepLength * 0.5;
        }

        vec2 offset = isHorizontal ? vec2(view.x, 0.0) : vec2(0.0, view.y);

        vec2 uv1 = currentUv - offset;
        vec2 uv2 = currentUv + offset;

        float lumaEnd1 = GetLuminance(pow(texture2D(colortex4, uv1).rgb, vec3(1.0f / 2.2f)));
        float lumaEnd2 = GetLuminance(pow(texture2D(colortex4, uv2).rgb, vec3(1.0f / 2.2f)));
			
        lumaEnd1 -= lumaLocalAverage;
        lumaEnd2 -= lumaLocalAverage;

        bool reached1 = abs(lumaEnd1) >= gradientScaled;
        bool reached2 = abs(lumaEnd2) >= gradientScaled;
        bool reachedBoth = reached1 && reached2;

        if (!reached1)
		{
            uv1 -= offset;
        }
        if (!reached2)
		{
            uv2 += offset;
        }

        if (!reachedBoth)
		{
            for (int i = 2; i < iterations; i++)
			{
                if (!reached1)
				{
                    lumaEnd1 = GetLuminance(pow(texture2D(colortex4, uv1).rgb, vec3(1.0f / 2.2f)));
                    lumaEnd1 = lumaEnd1 - lumaLocalAverage;
                }
                if (!reached2)
				{
				    lumaEnd2 = GetLuminance(pow(texture2D(colortex4, uv2).rgb, vec3(1.0f / 2.2f)));
                    lumaEnd2 = lumaEnd2 - lumaLocalAverage;
                }

                reached1 = abs(lumaEnd1) >= gradientScaled;
                reached2 = abs(lumaEnd2) >= gradientScaled;
                reachedBoth = reached1 && reached2;

                if (!reached1)
				{
                    uv1 -= offset * quality[i];
                }
                if (!reached2)
				{
                    uv2 += offset * quality[i];
                }

                if (reachedBoth) break;
            }
        }

        float distance1 = isHorizontal ? (TexCoords.x - uv1.x) : (TexCoords.y - uv1.y);
        float distance2 = isHorizontal ? (uv2.x - TexCoords.x) : (uv2.y - TexCoords.y);

        bool isDirection1 = distance1 < distance2;
        float distanceFinal = min(distance1, distance2);

        float edgeThickness = (distance1 + distance2);

        float pixelOffset = - distanceFinal / edgeThickness + 0.5;

        bool isLumaCenterSmaller = lumaCenter < lumaLocalAverage;

        bool correctVariation = ((isDirection1 ? lumaEnd1 : lumaEnd2) < 0.0) != isLumaCenterSmaller;

        float finalOffset = correctVariation ? pixelOffset : 0.0;

        float lumaAverage = (1.0 / 12.0) * (2.0 * (lumaDownUp + lumaLeftRight) + lumaLeftCorners + lumaRightCorners);
        float subPixelOffset1 = clamp(abs(lumaAverage - lumaCenter) / lumaRange, 0.0, 1.0);
        float subPixelOffset2 = (-2.0 * subPixelOffset1 + 3.0) * subPixelOffset1 * subPixelOffset1;
        float subPixelOffsetFinal = subPixelOffset2 * subPixelOffset2 * subpixelQuality;

        finalOffset = max(finalOffset, subPixelOffsetFinal);

        // Compute the final UV coordinates.
		vec2 finalUv = TexCoords;
        if (isHorizontal) {
            finalUv.y += finalOffset * stepLength;
        } else {
            finalUv.x += finalOffset * stepLength;
        }

        //color = texture2D(colortex4, finalUv).rgb;
        color = pow(texture2D(colortex4, finalUv).rgb, vec3(1.0f / 2.2f));
    }
}

// ACES Tone Mapping Function
vec3 toneMappingACES(const vec3 x) {
    float a = 2.51;
    float b = 0.03;
    float c = 2.43;
    float d = 0.59;
    float e = 0.14;
    return clamp((x * (a * x + b)) / (x * (c * x + d) + e), 0.0, 1.0);
}

vec3 calculateBloom(vec2 TexCoords)
{
    vec3 color = texture(colortex4, TexCoords).rgb;
    float brightness = max(max(color.r, color.g), color.b);
    vec3 bloom = max(color - BLOOM_THRESHOLD, 0.0) * BLOOM_INTENSITY;
    return bloom;
}

void main() {
    vec3 color = pow(texture2D(colortex4, TexCoords).rgb, vec3(1.0f / 2.2f));

	#if ANTI_ALIASING == 1
	FXAA311(color);
    #endif
	
    // Apply bloom
	//vec3 bloomColor = vec3(0.0);
	//#if BLOOM == 1
	//vec2 texelSize = 1.0 / textureSize(colortex4, 0);
    //for (int i = -BLUR_RADIUS; i <= BLUR_RADIUS; ++i) {
    //    bloomColor += calculateBloom(TexCoords + vec2(float(i) * texelSize.x, 0.0));
	//	bloomColor += calculateBloom(TexCoords + vec2(0.0, float(i) * texelSize.y));
    //}
	//bloomColor /= float(2 * BLUR_RADIUS + 1);
	//bloomColor = pow(bloomColor, vec3(1.0f / 2.2f));
    //#endif
    //color += bloomColor;

	//color = toneMappingACES(color);
	
    // Apply saturation, brightness, and contrast
    float average = (color.r + color.g + color.b) / 3.0;
    color.rgb = mix(vec3(average), color.rgb, SATURATION);
    color.rgb += BRIGHTNESS;
    color.rgb = ((color.rgb - 0.5) * CONTRAST) + 0.5;

    gl_FragColor = vec4(color, 1.0);
}
